@@ -12,7 +12,7 @@
  *
  * Also finds out IP of specified MAC
  *
- * $Id: arping.c 1210 2005-01-28 13:22:59Z marvin $
+ * $Id: arping.c 1893 2007-07-09 22:23:28Z marvin $
  */
 /*
  *  Copyright (C) 2000-2002 Thomas Habets <thomas@habets.pp.se>
@@ -58,6 +58,10 @@
 #include "win32.h"
 #include "win32/getopt.h"
 #endif
+
+#if !defined(linux)
+#define HAVE_WEIRD_BSD 1
+#endif
  
 #if defined(linux)
 #define HAVE_ESIZE_TYPES 1
@@ -89,7 +93,7 @@
 #define IP_ALEN 4
 #endif
 
-const float version = 2.05f;
+const float version = 2.06f;
 
 static libnet_t *libnet = 0;
 
@@ -230,10 +234,10 @@ static const char *arping_lookupdev(u_int32_t srcip, u_int32_t dstip,
 
 
 #ifdef WIN32
-static BOOL WINAPI arping_console_ctrl_handler(DWORD dwCtrlType )
+static BOOL WINAPI arping_console_ctrl_handler(DWORD dwCtrlType)
 {
 	if(verbose) {
-		printf("arping_console_ctrl_handler( %d )\n", dwCtrlType );
+		printf("arping_console_ctrl_handler( %d )\n", (int)dwCtrlType);
 	}
 	time_to_die = 1;
 
@@ -259,6 +263,56 @@ static void sigint(int i)
 	time_to_die = 1;
 }
 
+static void
+extended_usage()
+{
+	printf("\nOptions:\n");
+	printf("\n"
+	       "    -0     Use this option to ping with source IP address 0.0.0.0. Use this\n"
+	       "           when you haven't configured your interface yet.  Note that  this\n"
+	       "           may  get  the  MAC-ping  unanswered.   This  is  an alias for -S\n"
+	       "           0.0.0.0.\n"
+	       "    -a     Audiable ping.\n"
+	       "    -A     Only count addresses matching  requested  address  (This  *WILL*\n"
+	       "           break  most things you do. Only useful if you are arpinging many\n"
+	       "           hosts at once. See arping-scan-net.sh for an example).\n"
+	       "    -b     Like -0 but source broadcast source  address  (255.255.255.255).\n"
+	       "           Note that this may get the arping unanswered since it's not nor-\n"
+	       "           mal behavior for a host.\n"
+	       "    -B     Use instead of host if you want to address 255.255.255.255.\n"
+	       "    -c count\n"
+	       "           Only send count requests.\n"
+	       "    -d     Find duplicate replies.\n"
+	       "    -F     Don't try to be smart about the interface name.  (even  if  this\n"
+	       "           switch is not given, -i overrides smartness.\n"
+	       "    -h     Displays a help message and exits.\n"
+	       "    -i interface\n"
+	       "           Use the specified interface.\n"
+	       "    -q     Does not display messages, except error messages.\n"
+	       "    -r     Raw output: only the MAC/IP address is displayed for each reply.\n"
+	       "    -R     Raw output: Like -r but shows \"the other one\", can  be  combined\n"
+	       "           with -r.\n"
+	       "    -s MAC Set source MAC address. You may need to use -p with this.\n"
+	       "    -S IP  Like  -b and -0 but with set source address.  Note that this may\n"
+	       "       	   get the arping unanswered if the target does not have routing to\n"
+	       "           the  IP.  If you don't own the IP you are using, you may need to\n"
+	       "           turn on promiscious mode on the interface (with -p).  With  this\n"
+	       "           switch  you can find out what IP-address a host has without tak-\n"
+	       "           ing an IP-address yourself.\n"
+	       "    -t MAC Set target MAC address to use when pinging IP address.\n"
+	       "    -T IP  Use -T as target address when pinging MACs that won't respond to\n"
+	       "           a broadcast ping but perhaps to a directed broadcast.\n"
+	       "           Example:\n"
+	       "           To check the address of MAC-A, use knowledge of MAC-B and  IP-B.\n"
+	       "           $ arping -S <IP-B> -s <MAC-B> -p <MAC-A>\n"
+	       "    -p     Turn  on  promiscious  mode  on interface, use this if you don't\n"
+	       "           \"own\" the MAC address you are using.\n"
+	       "    -u     Show index=received/sent instead  of  just  index=received  when\n"
+	       "           pinging MACs.\n"
+	       "    -v     Verbose output. Use twice for more messages.\n"
+	       "    -w     (arping 2.x only) Time to wait between pings, in microseconds.\n");
+}
+
 /*
  *
  */
@@ -271,6 +325,9 @@ static void usage(int ret)
 	       "              [ -s <MAC> ] [ -t <MAC> ] [ -c <count> ] "
 	       "[ -i <interface> ]\n"
 	       "              <host/ip/MAC | -B>\n");
+#ifdef WIN32
+	extended_usage();
+#endif
 	exit(ret);
 }
 
@@ -670,22 +727,12 @@ static void pingmac_recv(const char *unused, struct pcap_pkthdr *h,
 }
 
 
-/*
- * 
- */
-static void ping_recv(pcap_t *pcap,u_int32_t packetwait, pcap_handler func)
+#ifdef WIN32
+static void
+ping_recv_win32(pcap_t *pcap,u_int32_t packetwait, pcap_handler func)
 {
        struct timeval tv,tv2;
        char done = 0;
-#ifndef WIN32
-       fd_set fds;
-#endif
-
-       if(verbose>3) {
-               printf("arping: receiving packets...\n");
-       }
-
-#ifdef WIN32
        /* windows won't let us do select() */
        if (-1 == gettimeofday(&tv2,NULL)) {
 	       fprintf(stderr, "arping: gettimeofday(): %s\n",
@@ -720,88 +767,124 @@ static void ping_recv(pcap_t *pcap,u_int32_t packetwait, pcap_handler func)
 		       done=1;
 	       }
        }
-#else
-	tv.tv_sec = packetwait / 1000000;
-	tv.tv_usec = packetwait % 1000000;
-
-	for (;!done;) {
-		int sr;
-		FD_ZERO(&fds);
-		FD_SET(pcap_fileno(pcap), &fds);
-		
-		if (-1 == gettimeofday(&tv2,NULL)) {
-			fprintf(stderr, "arping: "
-				"gettimeofday(): %s\n",
-				strerror(errno));
-			sigint(0);
-		}
-//		printf("running select()\n");
-
-		switch(sr = select(pcap_fileno(pcap)+1,
-				   &fds,
-				   NULL,NULL,&tv)) {
-		case -1:
-			if (errno == EINTR) {
-				return;
-			}
-			fprintf(stderr, "arping: select(): "
-				"%s\n", strerror(errno));
-			sigint(0);
-		case 0:
-			done = 1;
-			break;
-		default: {
-			int ret;
-			if (1 != (ret = pcap_dispatch(pcap, 1,
-						      func,
-						      NULL))) {
-				// rest, so we don't take 100% CPU... mostly
-				// hmm... does usleep() exist everywhere?
-				usleep(10);
-#ifndef HAVE_WEIRD_BSD
-				// weird is normal on bsd :)
-				if (verbose) {
-					fprintf(stderr, "arping: select=%d "
-						"pcap_dispatch=%d!\n",
-						sr, ret);
-				}
+}
 #endif
-			}
-			break; }
-		}
-		
-		if (-1 == gettimeofday(&tv,NULL)) {
-			fprintf(stderr, "arping: "
-				"gettimeofday(): %s\n",
-				strerror(errno));
-			sigint(0);
-		}
-		/*
-		 * setup next timeval, not very exact
-		 */
-		tv.tv_sec  = (packetwait / 1000000)
-			- (tv.tv_sec - tv2.tv_sec);
-		tv.tv_usec = (packetwait % 1000000)
-			- (tv.tv_usec - tv2.tv_usec);
-		while (tv.tv_usec < 0) {
-			tv.tv_sec--;
-			tv.tv_usec += 1000000;
-		}
-		if (tv.tv_sec < 0) {
-			tv.tv_sec = tv.tv_usec = 0;
-		}
-		
+
+static void
+fixup_timeval(struct timeval *tv)
+{
+	while (tv->tv_usec < 0) {
+		tv->tv_sec--;
+		tv->tv_usec += 1000000;
 	}
-//	if (tv.tv_usec == 0) {
-//		tv.tv_usec = 1;
-//	}
-	if (-1 == select(0, NULL,NULL,NULL, &tv)) {
-		if (errno == EINTR) {
-			return;
-		}
-		fprintf(stderr, "arping: select(delay): %s\n",strerror(errno));
+}
+
+
+static void
+gettv(struct timeval *tv)
+{
+	if (-1 == gettimeofday(tv,NULL)) {
+		fprintf(stderr, "arping: "
+			"gettimeofday(): %s\n",
+			strerror(errno));
 		sigint(0);
 	}
+}
+
+
+/*
+ * 
+ */
+static void
+ping_recv_unix(pcap_t *pcap,u_int32_t packetwait, pcap_handler func)
+{
+       struct timeval tv;
+       struct timeval endtime;
+       char done = 0;
+       fd_set fds;
+
+       gettv(&tv);
+       endtime.tv_sec = tv.tv_sec + (packetwait / 1000000);
+       endtime.tv_usec = tv.tv_usec + (packetwait % 1000000);
+       fixup_timeval(&endtime);
+
+       for (;!done;) {
+	       int sr;
+	       FD_ZERO(&fds);
+	       FD_SET(pcap_fileno(pcap), &fds);
+
+	       gettv(&tv);
+	       tv.tv_sec = endtime.tv_sec - tv.tv_sec;
+	       tv.tv_usec = endtime.tv_usec - tv.tv_usec;
+	       fixup_timeval(&tv);
+	       if (tv.tv_sec < 0) {
+		       tv.tv_sec = 0;
+		       tv.tv_usec = 1;
+		       done = 1;
+	       }
+	       if (time_to_die) {
+		       return;
+	       }
+#ifndef HAVE_WEIRD_BSD
+	       switch((sr = select(pcap_fileno(pcap)+1,
+				   &fds,
+				   NULL,NULL,&tv))) {
+	       case -1:
+		       if (errno == EINTR) {
+			       return;
+		       }
+		       fprintf(stderr, "arping: select(%lu.%lu): "
+			       "%s\n",
+			       tv.tv_sec,
+			       tv.tv_usec,
+			       strerror(errno));
+		       sigint(0);
+	       case 0:
+		       done = 1;
+		       break;
+	       default: {
+#else
+	       usleep(10);
+	       {{
+#endif
+		       int ret;
+		       if (1 != (ret = pcap_dispatch(pcap, 1,
+						     func,
+						     NULL))) {
+			       // rest, so we don't take 100% CPU... mostly
+			       // hmm... does usleep() exist everywhere?
+			       usleep(10);
+#ifndef HAVE_WEIRD_BSD
+			       // weird is normal on bsd :)
+			       if (verbose) {
+				       fprintf(stderr, "arping: select=%d "
+					       "pcap_dispatch=%d!\n",
+					       sr, ret);
+			       }
+		       }
+		       break;
+#else
+	               }
+#endif
+		       }
+	       }
+       }
+}
+
+/*
+ * 
+ */
+static void
+ping_recv(pcap_t *pcap,u_int32_t packetwait, pcap_handler func)
+{
+       if(verbose>3) {
+               printf("arping: receiving packets...\n");
+       }
+
+#ifdef WIN32
+       ping_recv_win32(pcap,packetwait,func);
+#else
+       ping_recv_unix(pcap,packetwait,func);
 #endif
 }
 
@@ -974,7 +1057,6 @@ int main(int argc, char **argv)
 	 * Handle dstip_given instead of ip address after parms (-B really)
 	 */
 	if (mode == NONE) {
-		unsigned char n[6];
 		if (optind + 1 == argc) {
 			mode = is_mac_addr(parm)?PINGMAC:PINGIP;
 		} else if (dstip_given) {
@@ -1140,7 +1222,7 @@ int main(int argc, char **argv)
 		}
 	}
 #ifdef WIN32
-	SetConsoleCtrlHandler(NULL, TRUE);
+	//SetConsoleCtrlHandler(NULL, TRUE);
 	SetConsoleCtrlHandler(arping_console_ctrl_handler, TRUE);
 #else
 	signal(SIGINT, sigint);
